use env_logger::Builder;
use log::{error, info};
use martillo_maldito::{cli::Cli, login_attempt::LoginAttempt, martillo_maldito::MartilloMaldito};
use std::{
    collections::HashMap,
    io::BufRead,
    process::{Child, Command, Stdio},
    thread::{sleep, spawn},
    time::Duration,
};
use structopt::StructOpt;

#[tokio::main]
async fn main() {
    match Cli::from_args() {
        Cli::BanService => {
            start_logger();

            if let Err(err) = start_ban_service().await {
                error!(err = err.to_string().as_str();
                    "Ban service"
                );
            }
        }
        Cli::GetBannedIps { docker } => {
            let banned_ips = MartilloMaldito::ipv4(docker).get_banned_ips();
            println!("{}", serde_json::to_string(&banned_ips).unwrap());
        }
        Cli::ListSecuredPorts { docker } => {
            let secured_ports = MartilloMaldito::ipv4(docker).get_secured_ports();
            println!("{}", serde_json::to_string(&secured_ports).unwrap());
        }
        Cli::MapSecuredPortsAllowedIps { docker } => {
            let secured_ports_with_allowed_ips =
                MartilloMaldito::ipv4(docker).get_secured_ports_with_allowed_ips();
            println!(
                "{}",
                serde_json::to_string(&secured_ports_with_allowed_ips).unwrap()
            );
        }
        Cli::IsPortSecured { port, docker } => {
            let is_port_secured = MartilloMaldito::ipv4(docker).is_port_secured(port);
            println!("{}", is_port_secured);
        }
        Cli::SecurePort {
            port,
            position,
            docker,
        } => {
            let port_secured = MartilloMaldito::ipv4(docker).secure_port(port, position);
            println!("{}", port_secured.is_ok())
        }
        Cli::UnsecurePort { port, docker } => {
            let port_unsecured = MartilloMaldito::ipv4(docker).unsecure_port(port);
            println!("{}", port_unsecured.is_ok())
        }
        Cli::AllowIpForPort {
            ip,
            port,
            position,
            docker,
        } => {
            let allowed_ip = MartilloMaldito::ipv4(docker).allow_ip_for_port(&ip, port, position);
            println!("{}", allowed_ip.is_ok())
        }
        Cli::RemoveAllowIpPort { ip, port, docker } => {
            let removed_allow_ip =
                MartilloMaldito::ipv4(docker).remove_allow_ip_for_port(&ip, port);
            println!("{}", removed_allow_ip.is_ok())
        }
        Cli::SaveRules => {
            println!("{}", MartilloMaldito::save_rules().is_ok())
        }
    }
}

async fn start_ban_service() -> std::io::Result<()> {
    let seconds_iptables = Duration::from_secs(60);
    info!(every_seconds = seconds_iptables.as_secs();
        "Saving IPTables"
    );

    spawn(move || loop {
        sleep(seconds_iptables);
        MartilloMaldito::save_rules().expect("Failed to save rules");
    });

    let child: Child = Command::new("journalctl")
        .arg("-D")
        .arg("/var/log/journal")
        .arg("-u")
        .arg("ssh")
        .arg("-f")
        .stdout(Stdio::piped())
        .spawn()
        .expect("Failed to start journalctl");

    let stdout = child.stdout.expect("Failed to capture stdout");
    let mut reader = std::io::BufReader::new(stdout);

    let martillo_maldito = MartilloMaldito::ipv4(false);
    let mut login_attempts: HashMap<String, usize> = HashMap::new();

    loop {
        let mut line = String::new();
        if let Err(err) = reader.read_line(&mut line) {
            error!(err = err.to_string().as_str();
                "Reading line"
            );
            continue;
        }

        if line.is_empty() {
            continue;
        }

        if let Some(login_attempt) = LoginAttempt::capture(&line) {
            info!(ip = login_attempt.ip.as_str(),
                user = login_attempt.user.as_str();
                "Login attempt",
            );

            match login_attempts.get_mut(&login_attempt.ip) {
                Some(count) => {
                    *count += 1;

                    if *count == 3 {
                        if martillo_maldito.ban_ip(&login_attempt.ip).is_ok() {
                            info!(ip = login_attempt.ip.as_str();
                                "Banned IP"
                            );
                        }

                        login_attempts.remove(&login_attempt.ip);
                    }
                }
                None => {
                    login_attempts.insert(login_attempt.ip, 1);
                }
            }
        }
    }
}

fn start_logger() {
    Builder::from_default_env().init();
}
